import time
from selenium import webdriver
import urllib.request
from urllib.parse import urlparse
import os
from quick_crawler.page import *

def get_domain(url):
    domain = urlparse(url).netloc
    return domain

def get_html_str_with_browser(url,driver_path="chromedriver.exe",implicitly_wait=0.5,root_ele="html",slient=False, wait_seconds=-1):

    if not os.path.exists(driver_path):
        print("Please set browser driver path in this function, versions between driver and browser must be same!")
        return

    # set chromedriver.exe's path
    if slient:
        options = webdriver.ChromeOptions()
        options.add_argument("--log-level=3")
        options.headless = True
        driver = webdriver.Chrome(executable_path=driver_path,
                                   chrome_options=options
                                  )
    else:
        driver = webdriver.Chrome(executable_path=driver_path
                                  )
    driver.implicitly_wait(implicitly_wait)
    # launch the page
    driver.get(url)

    html_obj = driver.find_element_by_tag_name(root_ele)
    if wait_seconds!=-1:
        time.sleep(5)

    html_str=html_obj.get_attribute("outerHTML")

    driver.close()
    return html_str

def get_page_meta(html_str):
    soup = BeautifulSoup(html_str, features="lxml")

    keywords=""
    description=""
    title=""
    if soup!=None:
        if soup.title!=None:
            title = soup.title.string
    else:
        return {
        "title":"",
        "lang":"",
        "keywords":"",
        "description":""
    }

    print('title = ', title)

    # print(soup.attrs)
    html=soup.find("html")
    if "lang" in html.attrs.keys():
        lang = html["lang"]
    else:
        lang = ""
    print("lang = ",lang)

    meta = soup.find_all('meta')
    # print(html_str)
    for tag in meta:
        if 'name' in tag.attrs.keys():
            name=tag.attrs['name'].strip().lower()
            if name=="description":
                description=tag.attrs['content']
            if name=="keywords":
                keywords=tag.attrs['content']
    if title==None:
        title=""
    if lang==None:
        lang=""
    if keywords==None:
        keywords=""
    if description!=None:
        description=""
    model = {
        "title":title.replace("\n",""),
        "lang":lang,
        "keywords":keywords.replace("\n",""),
        "description":description.replace("\n","")
    }
    return model



def check_url_ok(url,timeout=5):
    try:
        return urllib.request.urlopen(url,timeout=timeout).getcode() == 200
    except:
        return False

def revise_config_path(file_path,dict_kv):
    current_path = os.path.dirname(os.path.realpath(__file__))
    # spider_path = current_path + "/scrapy_projects/news_site/news_site/spiders/news_spider_template.py"
    script_text = open(file_path, 'r', encoding='utf-8').read()
    for k in dict_kv:
        script_text = script_text.replace(k, str(dict_kv[k]))

    spider_path_new = file_path.replace("_template", "")
    f_out = open(spider_path_new, 'w', encoding='utf-8')
    f_out.write(script_text)
    f_out.close()

def fetch_meta_info_from_sites(list_item,saved_folder="",download_time_out=30,dns_time_out=20,max_num_urls=1000,is_save_fulltext=False,use_plain_text=False,use_keywords=False):

    from pathlib import Path
    # list_result = page.quick_read_csv("datasets/news_sites.csv", fields=['Id', 'Title', 'Name', 'URL'])

    current_path = os.path.dirname(os.path.realpath(__file__))

    # url = list_result[330]
    for url in list_item:
        name = url[0]
        start_url = url[1]
        domain = get_domain(url[1])
        keywords=""
        if len(url)>=3:
            keywords=url[2]
        else:
            keywords=""
        print(start_url)
        # spider
        dict_kv = {
            "$NAME$": name,
            "$START_URL$":start_url,
            "$DOMAIN$":domain,
            "'$NAX_NUM_URLS$'":str(max_num_urls),
            "$DATA_ROOT$": str(saved_folder).replace("\\","/"),
            "$IS_SAVE_FULLTEXT$": str(is_save_fulltext),
            "$USE_PLAIN_TEXT$": str(use_plain_text),
            "$KEYWORDS$": str(keywords),
            "$USE_KEYWORDS$": str(use_keywords)
        }
        revise_config_path(current_path + "/scrapy_projects/news_site/news_site/spiders/news_spider_template.py",dict_kv=dict_kv)
        # settings
        dict_kv = {
            "'$DOWNLOAD_TIMEOUT$'": download_time_out,
            "'$DNS_TIMEOUT$'": dns_time_out
        }
        revise_config_path(current_path + "/scrapy_projects/news_site/news_site/settings_template.py",
                           dict_kv=dict_kv)

        current_path = os.path.dirname(os.path.realpath(__file__))
        # datasets_path=Path(current_path).parent.absolute()

        os.chdir(current_path+"/scrapy_projects/news_site")

        if saved_folder=="":
            os.system(f"scrapy crawl {name}")
        else:
            saved_path = f"{saved_folder}/{name}.csv"
            saved_path = saved_path.replace("\\", "/")
            print(saved_path)
            os.system(f"scrapy crawl {name} -o \"file:///{saved_path}\" -t csv")
        os.chdir(current_path)

import csv

def get_model_from_csv_file(file_path,fields):
    list_model=[]
    with open(file_path, newline='',encoding='utf-8') as csvfile:
        reader = csv.DictReader(csvfile)
        list_result = []
        for row in reader:
            list_model.append(row)
    return list_model

def summarize_downloaded_data(data_folder,save_path=""):
    list_model_all=[]
    for name in os.listdir(data_folder):
        name_folder=os.path.join(data_folder,name)
        for csv_file in os.listdir(name_folder):
            if csv_file.endswith(".csv"):
                csv_full_file=os.path.join(name_folder,csv_file)
                list_model=get_model_from_csv_file(csv_full_file,"title,lang,keywords,description,url,id".split(","))
                for idx,model in enumerate(list_model):
                    list_model[idx]["name"]=name
                    list_model[idx]["domain"]=csv_file.replace(".csv","")
                    list_model_all.append(list_model[idx])
    if save_path!="":
        quick_save_csv(save_path=save_path,field_names="name,domain,title,lang,keywords,description,url,id".split(","),list_rows=list_model_all)
    return list_model_all

def get_language_code_of_page(url):
    try:
        html_str=get_html_str_with_browser(url,driver_path="browsers/chromedriver.exe",slient=True)
        if html_str.strip()=="":
            return ""
        meta_model=get_page_meta(html_str)
        lang=meta_model["lang"]
        return lang
    except:
        return ""


import requests
import csv
import json
from bs4 import BeautifulSoup

def quick_html_page(url,headers=None,page_encoding='utf-8',save_file_path=None,save_encoding="utf-8"):
    if headers==None:
        headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36',
        }
    response = requests.get(url, headers=headers)
    html_str = response.content.decode(encoding=page_encoding)
    if save_file_path!=None:
        f_out = open(save_file_path, "w", encoding=save_encoding)
        f_out.write(str(html_str))
        f_out.close()
    return html_str

def quick_html_object(html_str):
    soup = BeautifulSoup(html_str, features="lxml")
    return soup

def quick_html_page_range(url,headers=None,page_encoding='utf-8',save_encoding="utf-8",save_file_path=None,min_page=1,max_page=100):
    list_html_str=[]
    for pi in range(min_page,max_page+1):
        current_url=url.replace("{pi}",str(pi))
        print(current_url)
        if save_file_path!=None:
            html_str=quick_html_page(current_url,headers,page_encoding, save_file_path=save_file_path.replace("{pi}",str(pi)), save_encoding=save_encoding)
        else:
            html_str = quick_html_page(current_url, headers, page_encoding)
        list_html_str.append(html_str)
    return list_html_str

def quick_remove_unicode(str,encoding='gbk',decoding='gbk'):
    string_encode = str.encode(encoding, "ignore")
    string_decode = string_encode.decode(decoding)
    return string_decode

def quick_save_csv(save_path,field_names,list_rows,encoding='utf-8'):
    with open(save_path, 'w', newline='',encoding=encoding) as csvfile:
        writer = csv.DictWriter(csvfile, fieldnames=field_names)
        writer.writeheader()
        for row in list_rows:
            dict_model = {}
            for f in field_names:
                dict_model[f]=row[f]
            writer.writerow(dict_model)

def quick_json_obj(url,headers=None,data=None):
    if headers==None:
        headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36',
        }
    if data!=None:
        x = requests.get(url, headers=headers, data=data)
    else:
        x = requests.get(url, headers=headers)
    # print(x.text)
    raw_str = quick_remove_unicode(x.text)
    pageObj = json.loads(raw_str)
    return pageObj

def quick_read_csv(csv_path,fields):
    with open(csv_path, newline='') as csvfile:
        reader = csv.DictReader(csvfile)
        list_result=[]
        for row in reader:
            l=[]
            for f in fields:
                l.append(row[f])
            list_result.append(l)
        return list_result

def quick_download_file(url,save_file_path,headers=None,):
    if headers!=None:
        headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36',
        }
    r = requests.get(url, headers=headers, stream=True)
    if r.status_code == 200:
        open(save_file_path, 'wb').write(r.content)  # 将内容写入图片

def quick_post_html_page(url,headers=None,data=None):
    if headers!=None:
        headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36',
        }
    if data!=None:
        r = requests.post(url, json=data,
                      headers=headers)
    else:
        r = requests.post(url)
    return r.text

def quick_post_json_obj(url,headers=None,data=None):
    if headers==None:
        headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36',
        }
    if data!=None:
        x = requests.post(url, headers=headers, data=data)
    else:
        x = requests.post(url, headers=headers)
    # print(x.text)
    raw_str = quick_remove_unicode(x.text)
    pageObj = json.loads(raw_str)
    return pageObj


import os
from semantickit.lang.wordnet import *
import urllib.request
from deep_translator import GoogleTranslator

lang_data='''
alpha3-b,alpha2,English
aar,aa,Afar
abk,ab,Abkhazian
afr,af,Afrikaans
aka,ak,Akan
alb,sq,Albanian
amh,am,Amharic
ara,ar,Arabic
arg,an,Aragonese
arm,hy,Armenian
asm,as,Assamese
ava,av,Avaric
ave,ae,Avestan
aym,ay,Aymara
aze,az,Azerbaijani
bak,ba,Bashkir
bam,bm,Bambara
baq,eu,Basque
bel,be,Belarusian
ben,bn,Bengali
bih,bh,Bihari languages
bis,bi,Bislama
bos,bs,Bosnian
bre,br,Breton
bul,bg,Bulgarian
bur,my,Burmese
cat,ca,Catalan; Valencian
cha,ch,Chamorro
che,ce,Chechen
chi,zh,Chinese
chu,cu,Church Slavic; Old Slavonic; Church Slavonic; Old Bulgarian; Old Church Slavonic
chv,cv,Chuvash
cor,kw,Cornish
cos,co,Corsican
cre,cr,Cree
cze,cs,Czech
dan,da,Danish
div,dv,Divehi; Dhivehi; Maldivian
dut,nl,Dutch; Flemish
dzo,dz,Dzongkha
eng,en,English
epo,eo,Esperanto
est,et,Estonian
ewe,ee,Ewe
fao,fo,Faroese
fij,fj,Fijian
fin,fi,Finnish
fre,fr,French
fry,fy,Western Frisian
ful,ff,Fulah
geo,ka,Georgian
ger,de,German
gla,gd,Gaelic; Scottish Gaelic
gle,ga,Irish
glg,gl,Galician
glv,gv,Manx
gre,el,"Greek, Modern (1453-)"
grn,gn,Guarani
guj,gu,Gujarati
hat,ht,Haitian; Haitian Creole
hau,ha,Hausa
heb,he,Hebrew
her,hz,Herero
hin,hi,Hindi
hmo,ho,Hiri Motu
hrv,hr,Croatian
hun,hu,Hungarian
ibo,ig,Igbo
ice,is,Icelandic
ido,io,Ido
iii,ii,Sichuan Yi; Nuosu
iku,iu,Inuktitut
ile,ie,Interlingue; Occidental
ina,ia,Interlingua (International Auxiliary Language Association)
ind,id,Indonesian
ipk,ik,Inupiaq
ita,it,Italian
jav,jv,Javanese
jpn,ja,Japanese
kal,kl,Kalaallisut; Greenlandic
kan,kn,Kannada
kas,ks,Kashmiri
kau,kr,Kanuri
kaz,kk,Kazakh
khm,km,Central Khmer
kik,ki,Kikuyu; Gikuyu
kin,rw,Kinyarwanda
kir,ky,Kirghiz; Kyrgyz
kom,kv,Komi
kon,kg,Kongo
kor,ko,Korean
kua,kj,Kuanyama; Kwanyama
kur,ku,Kurdish
lao,lo,Lao
lat,la,Latin
lav,lv,Latvian
lim,li,Limburgan; Limburger; Limburgish
lin,ln,Lingala
lit,lt,Lithuanian
ltz,lb,Luxembourgish; Letzeburgesch
lub,lu,Luba-Katanga
lug,lg,Ganda
mac,mk,Macedonian
mah,mh,Marshallese
mal,ml,Malayalam
mao,mi,Maori
mar,mr,Marathi
may,ms,Malay
mlg,mg,Malagasy
mlt,mt,Maltese
mon,mn,Mongolian
nau,na,Nauru
nav,nv,Navajo; Navaho
nbl,nr,"Ndebele, South; South Ndebele"
nde,nd,"Ndebele, North; North Ndebele"
ndo,ng,Ndonga
nep,ne,Nepali
nno,nn,"Norwegian Nynorsk; Nynorsk, Norwegian"
nob,nb,"Bokmål, Norwegian; Norwegian Bokmål"
nor,no,Norwegian
nya,ny,Chichewa; Chewa; Nyanja
oci,oc,Occitan (post 1500)
oji,oj,Ojibwa
ori,or,Oriya
orm,om,Oromo
oss,os,Ossetian; Ossetic
pan,pa,Panjabi; Punjabi
per,fa,Persian
pli,pi,Pali
pol,pl,Polish
por,pt,Portuguese
pus,ps,Pushto; Pashto
que,qu,Quechua
roh,rm,Romansh
rum,ro,Romanian; Moldavian; Moldovan
run,rn,Rundi
rus,ru,Russian
sag,sg,Sango
san,sa,Sanskrit
sin,si,Sinhala; Sinhalese
slo,sk,Slovak
slv,sl,Slovenian
sme,se,Northern Sami
smo,sm,Samoan
sna,sn,Shona
snd,sd,Sindhi
som,so,Somali
sot,st,"Sotho, Southern"
spa,es,Spanish; Castilian
srd,sc,Sardinian
srp,sr,Serbian
ssw,ss,Swati
sun,su,Sundanese
swa,sw,Swahili
swe,sv,Swedish
tah,ty,Tahitian
tam,ta,Tamil
tat,tt,Tatar
tel,te,Telugu
tgk,tg,Tajik
tgl,tl,Tagalog
tha,th,Thai
tib,bo,Tibetan
tir,ti,Tigrinya
ton,to,Tonga (Tonga Islands)
tsn,tn,Tswana
tso,ts,Tsonga
tuk,tk,Turkmen
tur,tr,Turkish
twi,tw,Twi
uig,ug,Uighur; Uyghur
ukr,uk,Ukrainian
urd,ur,Urdu
uzb,uz,Uzbek
ven,ve,Venda
vie,vi,Vietnamese
vol,vo,Volapük
wel,cy,Welsh
wln,wa,Walloon
wol,wo,Wolof
xho,xh,Xhosa
yid,yi,Yiddish
yor,yo,Yoruba
zha,za,Zhuang; Chuang
zul,zu,Zulu
'''

def get_language_code3(code2):
    # current_path = os.path.dirname(os.path.realpath(__file__))
    # lines=open(current_path+"/data/language-codes-3b2.csv",'r',encoding='utf-8').readlines()
    lines=lang_data.split("\n")
    dict_lang={}
    for line in lines:
        ls=line.strip().split(",")
        if len(ls)<2:
            continue
        dict_lang[ls[1]]=ls[0]
    if code2 in dict_lang:
        return dict_lang[code2]
    else:
        return None

def get_language_code_list():
    # current_path = os.path.dirname(os.path.realpath(__file__))
    # lines=open(current_path+"/data/language-codes-3b2.csv",'r',encoding='utf-8').readlines()
    lines=lang_data.split("\n")
    list_lang=[]
    for line in lines:
        ls=line.strip().split(",")
        if len(ls)<2:
            continue
        list_lang.append(ls[1])
    return list_lang

def get_language_code2(code3):
    # current_path = os.path.dirname(os.path.realpath(__file__))
    # lines=open(current_path+"/data/language-codes-3b2.csv",'r',encoding='utf-8').readlines()
    lines=lang_data.split("\n")
    dict_lang={}
    for line in lines:
        ls=line.strip().split(",")
        if len(ls)<2:
            continue
        dict_lang[ls[0]]=ls[1]
    if code3 in dict_lang:
        return dict_lang[code3]
    else:
        return None

def get_lang3(lang):
    if '-' in lang:
        lang=lang.split("-")[0]
    if '_' in lang:
        lang=lang.split("_")[0]
    return get_language_code3(lang)

def get_lang2(lang):
    if '-' in lang:
        lang=lang.split("-")[0]
    return get_language_code2(lang)



def get_all_related_words(text):
    nltk.download("wordnet")
    nltk.download('omw')
    dict_lang_all = get_all_related_word_from_text(text)
    print()
    for lang in dict_lang_all:
        print(lang, ','.join(dict_lang_all[lang]))
    return dict_lang_all

def get_all_related_words_by_lang(dict_lang_all,lang):
    if lang in dict_lang_all:
        return dict_lang_all[lang]
    else:
        return None



def get_translation(text, target):
    try:
        translated = GoogleTranslator(source='auto', target=target).translate(text)
        # print(translated)
        return translated
    except:
        return None

def get_lang_dict_by_translation(src_lang, to_translate):
    list_lang = get_language_code_list()
    dict_lang = {}
    dict_lang[src_lang] = to_translate
    for lang in list_lang:
        if lang != src_lang:
            translated = get_translation(to_translate, lang)
            if translated != None:
                dict_lang[lang] = translated
            print(lang, " -> ", translated)
    return dict_lang



import scrapy
from scrapy.spiders import CrawlSpider, Rule
from scrapy.linkextractors import LinkExtractor
from bs4 import BeautifulSoup
from scrapy.exceptions import CloseSpider
from quick_crawler import page
import hashlib
import os
MAX_NUM_URLS=100
DATA_ROOT='D:/UIBE科研/国自科青年/开源项目/quick-crawler/examples/news_data1'
IS_SAVE_FULLTEXT='True'
USE_PLAIN_TEXT='False'
KEYWORDS='digitale Wirtschaft'
USE_KEYWORDS='True'
url_count=0
list_keywords=KEYWORDS.split(";")
list_keywords=[k.strip().lower() for k in list_keywords]

def get_md5_url(url):
    url=str(url)
    m = hashlib.md5()
    m.update(url.encode('utf-8'))
    return m.hexdigest()

def get_page_meta(html_str):
    soup = BeautifulSoup(html_str, features="lxml")

    keywords=""
    description=""

    title = soup.title.string
    print('title = ', title)

    # print(soup.attrs)
    html=soup.find("html")
    if "lang" in html.attrs.keys():
        lang = html["lang"]
    else:
        lang = ""
    print("lang = ",lang)

    meta = soup.find_all('meta')
    # print(html_str)
    for tag in meta:
        if 'name' in tag.attrs.keys():
            name=tag.attrs['name'].strip().lower()
            if name=="description":
                description=tag.attrs['content']
            if name=="keywords":
                keywords=tag.attrs['content']
    model = {
        "title":title.replace("\n",""),
        "lang":lang,
        "keywords":keywords.replace("\n",""),
        "description":description.replace("\n","")
    }
    return model

def get_body(html):
    if html != None:
        body = html.find("body")
    else:
        body = None
    return body

class NewsSpider(CrawlSpider):
    name = 'AUT/derstandard.at'
    allowed_domains = ['derstandard.at']
    start_urls = ['https://derstandard.at/']

    rules = (
        Rule(LinkExtractor(), callback='parse_item', follow=True),
    )

    def parse_item(self, response):
        global url_count
        global list_model
        url_count+=1
        print(self.name + "'s URL: ", response.url)
        html_str=response.text
        html=BeautifulSoup(html_str,features='lxml')
        remove_tags=['style','script','svg','path','noscript']
        for tag in remove_tags:
            for item in html.findAll(tag):
                item.decompose()
        # detect if exists keywords
        if USE_KEYWORDS=="True":
            if len(list_keywords)!=0:
                body=get_body(html)
                flag_find=False
                for k in list_keywords:
                    if body!=None and k in body.text:
                        flag_find=True
                        break
                if flag_find==False:
                    # stop the call back
                    return None

        html_str=str(html.find("html"))
        if IS_SAVE_FULLTEXT=="True":
            name_folder=DATA_ROOT+"/"+self.name
            if not os.path.exists(name_folder):
                os.mkdir(name_folder)
            url_id=get_md5_url(response.url)
            url_file_path=name_folder+"/"+url_id+".txt"
            # if url has been downloaded, ignore
            if not os.path.exists(url_file_path):
                f_out=open(url_file_path,'w',encoding='utf-8')
                if USE_PLAIN_TEXT=="True":
                    body=get_body(html)
                    if body!=None:
                        f_out.write(body.text)
                else:
                    f_out.write(html_str)
                f_out.close()
                f_out = open(name_folder + "/" + url_id + "_url.txt",'w',encoding='utf-8')
                f_out.write(response.url)
                f_out.close()

        meta_model=get_page_meta(html_str)
        meta_model["url"]=page.quick_remove_unicode(response.url)
        meta_model["id"]=page.quick_remove_unicode(self.name)
        print(meta_model)
        #list_model.append(meta_model)
        if url_count>=int(MAX_NUM_URLS):
            raise CloseSpider('termination condition met')
        return meta_model


